"""
This module exports routines of PHCpack to work with Newton polytopes.
"""

def random_points(dim, nbr, low, upp):
    r"""
    Generates a list of random integer points.
    Returns a list of *nbr* points of dimension *dim*,
    with integer coordinates in the range from *low* to *upp*.
    """
    from random import randint
    result = []
    for _ in range(nbr):
        coords = [randint(low, upp) for _ in range(dim)]
        point = tuple(coords)
        result.append(point)
    return result

def support(nvr, pol):
    r"""
    The support of a multivariate polynomial is a set of exponents
    of the monomials that appear with nonzero coefficient.
    Given in *nvr* the number of variables and in *pol* a string
    representation of a polynomial in *nvr* variables,
    returns the support of the polynomial as a list of tuples.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_standard_Laurent_system
    from phcpy.phcpy2c3 \
    import py2c_syscon_initialize_number_of_standard_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_store_standard_Laurential
    from phcpy.phcpy2c3 import py2c_scan_for_symbols
    py2c_syscon_clear_standard_Laurent_system()
    py2c_syscon_initialize_number_of_standard_Laurentials(nvr)
    nchar = len(pol)
    dim = py2c_scan_for_symbols(nchar, pol)
    if(dim <= nvr):
        fail = py2c_syscon_store_standard_Laurential(nchar, nvr, 1, pol)
    else:
        print('WARNING:', nvr, 'is smaller than', dim, end=' ')
        print('the number of symbols in', pol)
        print('Setting the number of variables to', dim, '...')
        fail = py2c_syscon_store_standard_Laurential(nchar, dim, 1, pol)
    if(fail != 0):
        return fail
    else:
        # from phcpy.phcpy2c3 import py2c_syscon_number_of_Laurent_terms
        from phcpy.phcpy2c3 import py2c_giftwrap_support_size
        from phcpy.phcpy2c3 import py2c_giftwrap_support_string
        from phcpy.phcpy2c3 import py2c_giftwrap_clear_support_string
        # ntm = py2c_syscon_number_of_Laurent_terms(1)
        # print 'number of terms : %d' % ntm
        size = py2c_giftwrap_support_size(1) # take the first polynomial
        # print 'size of support :', size
        supp = py2c_giftwrap_support_string(size)
        # print 'support string :', supp
        py2c_giftwrap_clear_support_string()
        result = eval(supp)
        return result

def support_sets(pols, **nbvar):
    r"""
    Returns the support sets of all polynomials in the list *pols*.
    The *nbvar* is the number of variables in the system.
    if *nbvar* is omitted, then the system is assumed to be square.
    """
    from phcpy.interface import store_standard_laurent_system
    from phcpy.phcpy2c3 import py2c_syscon_clear_standard_Laurent_system
    from phcpy.phcpy2c3 import py2c_giftwrap_support_size
    from phcpy.phcpy2c3 import py2c_giftwrap_support_string
    from phcpy.phcpy2c3 import py2c_giftwrap_clear_support_string
    py2c_syscon_clear_standard_Laurent_system()
    if len(nbvar) == 0:
        store_standard_laurent_system(pols)
    else:
        nvr = nbvar.values()[0]
        store_standard_laurent_system(pols, nbvar=nvr)
    result = []
    for k in range(len(pols)):
        size = py2c_giftwrap_support_size(k+1)
        supp = py2c_giftwrap_support_string(size)
        result.append(eval(supp))
        py2c_giftwrap_clear_support_string()
    return result

def initial_form(pols, normal):
    r"""
    Returns the initial form of the polynomials in *pols*
    with respect to the inner normal with coordinates in *normal*.
    """
    from phcpy.phcpy2c3 import py2c_syscon_clear_standard_Laurent_system
    from phcpy.phcpy2c3 \
    import py2c_syscon_initialize_number_of_standard_Laurentials
    from phcpy.phcpy2c3 import py2c_syscon_store_standard_Laurential
    from phcpy.phcpy2c3 import py2c_syscon_load_standard_Laurential
    from phcpy.phcpy2c3 import py2c_giftwrap_initial_form
    py2c_syscon_clear_standard_Laurent_system()
    dim = max(len(pols), len(normal))
    py2c_syscon_initialize_number_of_standard_Laurentials(dim)
    for i in range(len(pols)):
        nchar = len(pols[i])
        fail = py2c_syscon_store_standard_Laurential(nchar, dim, i+1, pols[i])
    strnrm = str(tuple(normal))
    fail = py2c_giftwrap_initial_form(len(normal), len(strnrm), strnrm)
    result = []
    for i in range(len(pols)):
        result.append(py2c_syscon_load_standard_Laurential(i+1))
    return result

def initial_support(points, normal):
    r"""
    Returns the list of elements in *points* that make the minimal inner
    product with the given *normal(, as the second element in a tuple.
    The first element is the value of that minimal inner product.
    Every tuple in *points* must have the same length as *normal*.
    """
    inprod = lambda x, y: sum([a*b for (a, b) in zip(x, y)])
    vals = [inprod(point, normal) for point in points]
    minv = min(vals)
    idx = [ind for ind in range(len(vals)) if vals[ind] == minv]
    result = [points[ind] for ind in idx]
    return (minv, result)

def planar_hull_checkout(vertices, normals, verbose=True):
    r"""
    Given a list of *vertices* and a list of *normals* as output
    of a convex hull algorithm in the plane, this function checks
    whether the initial support of every normal consists of
    exactly two points that appear with consecutive indices
    (modulo the length of the list) in the list of *vertices*.
    Return True if the checks pass, False otherwise.
    """
    allpts = []
    for normal in normals:
        (val, inisup) = initial_support(vertices, normal)
        if verbose:
            print('normal', normal, 'has value', val)
            print('  supports', inisup)
        if(len(inisup) != 2):
            print('normal', normal, 'does not define an edge')
            return False
        else:
            adx = vertices.index(inisup[0])
            bdx = vertices.index(inisup[1])
            if(adx > bdx):
                (adx, bdx) = (bdx, adx)
            if verbose:
                print('  indices of support :', adx, bdx)
            if(adx + 1 != bdx):
                if(adx != 0):
                    print('indices', adx, bdx, 'are not consecutive')
                    return False
                elif(bdx != len(vertices)-1):
                    print('indices', adx, bdx, 'are not consecutive')
                    return False
            allpts.append(adx)
            allpts.append(bdx)
    if(len(allpts) == 2*len(vertices)):
        return True
    else:
        print('points on edges :', allpts)
        print('not all vertices are edges')
        return False

def convex_hull_checkin(dim, points):
    r"""
    Checks whether the input arguments satisfy the requirements:
    *points* is a list of tuples that each contain as many integer
    numbers as the value of *dim*.
    Returns True if the requirements are satisfied,
    returns False otherwise.
    """
    if not isinstance(points, list):
        print('the argument points is not a list')
        return False
    else:
        tup = [isinstance(x, tuple) for x in points]
        if(sum(tup) != len(points)):
            print('not every element in points is a tuple')
        else:
            for point in points:
                if(len(point) != dim):
                    print('the point', point, 'is not of length', dim)
                else:
                    coord = [isinstance(x, int) for x in point]
                    if(sum(coord) != dim):
                        print(point, 'contains non integer values')
    return True

def convex_hull_checkout(dim, points, facets, verbose=True):
    r"""
    Checks whether for each facet in the list of *facets*,
    the facet is supported on the *points* defined by the
    computed inner normal and the minimal value.
    Returns True if the check passes, returns False otherwise.
    """
    for facet in facets:
        if verbose:
            print('checking facet', facet)
        (calval, normal) = (facet[0], facet[1])
        (chkval, inisup) = initial_support(points, normal)
        if verbose:
            print('  supports :', inisup)
            print('  checked value :', chkval)
        if(calval != chkval):
            print('checked value', chkval, '!=', calval) 
            return False
        else:
            idx = [1 + points.index(x) for x in inisup]
            if verbose:
                print('  indices of support :', idx, '?=', facet[2])
            okay = True  # careful: not all points are vertices
            for fpt in facet[2]:
                if(not(fpt in idx)):
                    okay = False
                if(not okay):
                    break
            if verbose:
                print('  checked equalities :', okay)
            if(not okay):
                print('checked indices do not agree')
                return False
    return True

def vertices_in_facets(facets):
    r"""
    Given the list of *facets*, returns the list of indices
    to the vertices, to the points that span the facets.
    """
    result = []
    for facet in facets:
        pts = facet[2]
        for idx in pts:
            if(not (idx in result)):
                result.append(idx)
    return result

def edges_in_facets(facets):
    r"""
    Given the the list of *facets*, returns the list of tuples
    of indices to the point that span the edges of the facets. 
    """
    result = []
    for facet in facets:
        pts = facet[2]
        for idx in range(len(pts)-1):
            edge = [pts[idx], pts[idx+1]]
            edge.sort()
            tuped = tuple(edge)
            if(not(tuped in result)):
                result.append(tuped)
        edge = [pts[len(pts)-1], pts[0]]
        edge.sort()
        tuped = tuple(edge)
        if(not(tuped in result)):
            result.append(tuped)
    return result

def planar_convex_hull(points, checkin=True, checkout=True):
    r"""
    The convex hull of a point configuration in the plane
    consists of an ordered list of vertex *points*, ordered
    such that any two consecutive points span an edge,
    with the list of corresponding inner normals.
    If *checkin* (by default), the type of the input is checked.
    If *checkout* (by default), the output is checked.
    """
    if checkin:
        if not convex_hull_checkin(2, points):
            print('the input is not correct')
            return None
    from phcpy.phcpy2c3 import py2c_giftwrap_planar
    strpoints = str(points)
    strhull = py2c_giftwrap_planar(len(strpoints), strpoints)
    hull = eval(strhull)
    if checkout:
        if not planar_hull_checkout(hull[0], hull[1]):
            print('the output is not correct')
    return hull

def convex_hull(dim, points, checkin=True, checkout=True):
    r"""
    Returns the list of facets of the convex hull of the points,
    given in *points*.  The dimension of the ambient space is in *dim*.
    If *checkin* (by default), the type of the input is checked.
    If *checkout* (by default), the output is checked.
    """
    if checkin:
        if not convex_hull_checkin(dim, points):
            return None
    from phcpy.phcpy2c3 import py2c_giftwrap_convex_hull
    from phcpy.phcpy2c3 import py2c_giftwrap_number_of_facets
    from phcpy.phcpy2c3 import py2c_giftwrap_retrieve_facet
    strpoints = str(points)
    fail = py2c_giftwrap_convex_hull(len(strpoints), strpoints)
    nbrfacets = py2c_giftwrap_number_of_facets(dim)
    print('computed', nbrfacets, 'facets')
    result = []
    for k in range(nbrfacets):
        strfacet = py2c_giftwrap_retrieve_facet(dim, k)
        facet = eval(strfacet)
        result.append(facet)
    if(dim == 3):
        from phcpy.phcpy2c3 import py2c_giftwrap_clear_3d_facets
        fail = py2c_giftwrap_clear_3d_facets()
    if(dim == 4):
        from phcpy.phcpy2c3 import py2c_giftwrap_clear_4d_facets
        fail = py2c_giftwrap_clear_4d_facets()
    if checkout:
        if not convex_hull_checkout(dim, points, result):
            print('the list of facets is not correct')
    return result

def check_mixture(mixture, points):
    """
    The sum of the integers in the list mixture equal
    the dimension of each point in points.
    Returns True if the mixture type passes the test,
    otherwise, prints an error message and returns False.
    """
    if not isinstance(mixture, list):
        print('the argument mixture is not a list')
        return False
    if not isinstance(points, list):
        print('the argument points is not a list')
        return False
    if len(mixture) != len(points):
        print('mixture length is not equal to points length')
        return False
    dim = sum(mixture)
    for sup in points:
        for point in sup:
           if len(point) != dim:
              print('mixture does not match length of point')
              return False
    return True

def mixed_volume(mixture, points, checkin=True):
    r"""
    Returns the mixed volume of the list of lists in *points*.
    Both *mixture* and *points* have the same length.
    The list *mixture* counts the number of times each support
    in *points* should be counted.
    For example, to compute the volume of a three dimensional polytope,
    the *mixture* is [3].  In general, the *mixture* determines the powers
    of the unknowns in the Minkowski polynomial of which the computed
    mixed volume is its coefficient.
    If checkin, then the mixture will be tested to match the length
    of each point in points.
    Examples:
    >>> q1 = [(1, 1), (1, 0), (0, 1), (0, 0)]
    >>> q2 = [(2, 2), (1, 0), (0, 1)]
    >>> mv([1, 1], [q1, q2])
    4
    >>> mv([2], [q1])
    2
    """
    from phcpy.phcpy2c3 import py2c_celcon_initialize_supports as init
    from phcpy.phcpy2c3 import py2c_celcon_set_type_of_mixture as setmix
    from phcpy.phcpy2c3 import py2c_celcon_append_lifted_point as applft
    from phcpy.phcpy2c3 import py2c_celcon_mixed_volume_of_supports as mixvol
    if checkin:
        if not check_mixture(mixture, points):
            print('incorrect type of mixture')
            return -1
    nbr = len(mixture)
    init(nbr)
    setmix(nbr, str(mixture))
    for k in range(nbr):
        for point in points[k]:
            lpt = list(point)
            lpt.append(0)
            applft(len(lpt), k+1, str(lpt))
    return mixvol()

def test_planar_hull(nbr=7, size=9):
    r"""
    Generates a random point configuration in the plane
    and then computes its convex hull.
    By default, the number of points equals 7,
    in general it is the value of the parameter *nbr*.
    The range of the coordinates in the point is defined
    by the value of *size*, from -*size* to *size*.
    """
    pts = random_points(2, nbr, -size, size)
    print('the points :', pts)
    (vertices, normals) = planar_convex_hull(pts)
    print('the vertices :', vertices)
    print('inner normals :', normals)

def test_convex_hull(dim=3, nbr=10, size=9):
    r"""
    Generates a random point configuration in 3-space by default
    (although also *dim* = 4 works) and then computes its convex hull.
    By default, 10 points are generated, while in general,
    the number of points in the configurations equals *nbr*.
    The range of the coordinates in the point is defined
    by the value of *size*, from -*size* to *size*.
    """
    pts = random_points(dim, nbr, -size, size)
    print('the points :', pts)
    facets = convex_hull(dim, pts)
    vertices = vertices_in_facets(facets)
    edges = edges_in_facets(facets)
    print('vertices :', vertices)
    print('edges :', edges)
    print('the facets :')
    for facet in facets:
        print(facet)
    if(dim == 3):
        eultup = (len(facets), len(edges), len(vertices))
        eulsum = len(facets) - len(edges) + len(vertices)
        streul = '#facets - #edges + #vertices = ' \
               + '%d - %d + %d' % eultup \
               + ' = %d' % eulsum
        print(streul)

def integer_mixed_cell(dim, nbr, idx, verbose=True):
    r"""
    Given are three integers and one boolean, respectively:

    *dim*: the number of coordinates in the inner normal,

    *nbr*: the number of distinct supports,

    *idx*: the index to the cell (starts at one, instead of at zero), and

    *verbose*: the verbose flag.

    Returns the extracted data for the mixed cell with index *idx*.
    If verbose, the data is written to screen.
    """
    from ast import literal_eval
    from phcpy.phcpy2c3 import py2c_intcelcon_get_inner_normal as getnormal
    from phcpy.phcpy2c3 import py2c_intcelcon_mixed_volume as mixvol
    from phcpy.phcpy2c3 import py2c_intcelcon_number_of_points_in_cell as npts
    from phcpy.phcpy2c3 import py2c_intcelcon_get_point_in_cell as getpoint
    normal = literal_eval(getnormal(dim, idx))
    mv = mixvol(idx)
    lenpts = literal_eval(npts(idx, nbr))
    if verbose:
        print('inner normal :', normal)
        print('mixed volume :', mv)
        print('number of points :', lenpts)
    supp = [ [] for _ in range(nbr)]
    for i in range(nbr):  # scan the i-th support
        if verbose:
            print('points in support', i+1, ':')
        for j in range(lenpts[i]): # get j-th point in i-th support
            point = getpoint(dim, idx, i+1, j+1)
            if verbose:
                print(eval(point))
            supp[i].append(eval(point))
    return (normal, mv, tuple(supp))
        
def integer_mixed_cells(mixture, points, verbose=True):
    r"""
    Given a tuple of lifted support sets in *points*, computes all mixed cells
    in the regular subdivision defined by the integer lifting values
    given as the last coordinate of every point in the lifted supports.
    If *verbose*, then output is written to screen.
    Returns the mixed volume as the sum of the volumes of the cells.
    """
    from ast import literal_eval
    from phcpy.phcpy2c3 import py2c_intcelcon_set_type_of_mixture as setmix
    from phcpy.phcpy2c3 import py2c_intcelcon_type_of_mixture as getmix
    from phcpy.phcpy2c3 import py2c_intcelcon_initialize_supports as initsup
    from phcpy.phcpy2c3 import py2c_intcelcon_append_lifted_point as applpt
    from phcpy.phcpy2c3 import py2c_intcelcon_get_lifted_point as getlpt
    from phcpy.phcpy2c3 import py2c_intcelcon_length_of_supports as lensup
    from phcpy.phcpy2c3 import py2c_intcelcon_make_subdivision as makesub
    from phcpy.phcpy2c3 import py2c_intcelcon_number_of_cells as nbrcells
    from phcpy.phcpy2c3 import py2c_intcelcon_get_inner_normal as getnormal
    from phcpy.phcpy2c3 import py2c_intcelcon_mixed_volume as mixvol
    from phcpy.phcpy2c3 import py2c_intcelcon_write_mixed_cell_configuration
    setmix(len(mixture), str(mixture))
    if verbose:
        print('the type of mixture stored :', getmix())
    initsup(len(mixture))
    for k in range(len(points)):
        if verbose:
            print('points', k, points[k])
        for point in points[k]:
            if verbose:
                print('append lifted point :', point)
            applpt(len(point), k+1, str(point))
    lenpts = literal_eval(lensup())
    if verbose:
        dim = len(points[0][0])
        print('lengths of supports :', lenpts)
        for i in range(len(lenpts)):
            print('lifted points in support', i, ':')
            for j in range(lenpts[k]):
                print(getlpt(dim,i+1,j+1))
    makesub()
    if verbose:
        py2c_intcelcon_write_mixed_cell_configuration()
    number = nbrcells()
    totmv = 0
    if verbose:
        dim = len(points[0][0])
        print('number of cells :', number)
        for k in range(number):
            print('cell', k+1, 'has normal :', getnormal(dim, k+1), end='')
            mv = mixvol(k+1)
            print(' mixed volume :', mv)
            totmv = totmv + mv
    else:
        totmv = 0
        for k in range(number):
            totmv = totmv + mixvol(k+1)
    return totmv;

def test_integer_mixed_volume():
    """
    Tests mixed volume computation via integer valued lifting functions.
    """
    pts = ([[1, 0, 0, 1], [0, 1, 0, -1], [0, 0, 1, 2], [0, 0, 0, 0]], \
           [[2, 0, 1, 2], [1, 1, 1, 0], [0, 1, 2, 4], [2, 1, 0, 0], \
            [0, 0, 0, 3]]) 
    mv = integer_mixed_cells([1, 2], pts) 
    print('the mixed volume :', mv)
    dim = len(pts[0][0])
    firstcell = integer_mixed_cell(dim, len(pts), 1)
    print('the first cell :\n', firstcell)

def test_mixed_volume():
    """
    Runs some simple tests on mixed volume computation.
    """
    simple = [(0, 0, 0), (1, 0, 0), (0, 2, 0), (0, 0, 3)]
    mixvol = mixed_volume([3], tuple([simple]))
    print(simple, 'has volume', mixvol)
    points = random_points(3, 10, -9, 9)
    mixvol = mixed_volume([3], tuple([points]))
    print(points, 'has volume', mixvol)
    mixvol = mixed_volume([2, 1], (points, simple))
    print('the mixed volume of the two is', mixvol)
    cyclic5 = ([(1, 0, 0, 0, 0), (0, 1, 0, 0, 0), (0, 0, 1, 0, 0), \
                (0, 0, 0, 1, 0), (0, 0, 0, 0, 1)],
               [(1, 1, 0, 0, 0), (0, 1, 1, 0, 0), (0, 0, 1, 1, 0), \
                (0, 0, 0, 1, 1), (1, 0, 0, 0, 1)],
               [(1, 1, 1, 0, 0), (0, 1, 1, 1, 0), (0, 0, 1, 1, 1), \
                (1, 0, 0, 1, 1), (1, 1, 0, 0, 1)],
               [(1, 1, 1, 1, 0), (0, 1, 1, 1, 1), (1, 0, 1, 1, 1), \
                (1, 1, 0, 1, 1), (1, 1, 1, 0, 1)],
               [(1, 1, 1, 1, 1), (0, 0, 0, 0, 0)])
    mixvol = mixed_volume([1, 1, 1, 1, 1], cyclic5)
    print('the mixed volume of the cyclic 5-roots problem is', mixvol)

if __name__ == "__main__":
    # test_planar_hull()
    # test_convex_hull()
    test_integer_mixed_volume()



"""
Tests the retrieval of the version string of the PHCpack library,
using ctypes, in a platform independent manner.
The function getPHCmod() returns the library module for the three
supported platforms.  The functions int4a2str() and str2int4a()
convert integer arrays to strings and string to integer arrays,
using the ctypes string buffer type.
The function int4a2nbr() converts a list of integers to the
string buffer representation of the 32-bit integer array version.
"""
import ctypes
import sys
from ctypes import create_string_buffer
from ctypes import c_int32, c_double, pointer, sizeof
from struct import unpack

# relative location of the PHCpack library
LOCATION = "."

def get_phcfun_fromlib(vrblvl=0):
    """
    Returns the proper function according to the platform.
    For the correct execution, the file libPHCpack,
    with the extension .so, .dylib, or .dll must be present.
    """
    if vrblvl > 0:
        print('in get_phcfun_from_lib ...')
    if 'linux' in sys.platform:
        libphcpack = LOCATION + "/libPHCpack.so"
        phcpack = ctypes.CDLL(libphcpack)
        return phcpack._ada_use_c2phc
    if 'darwin' in sys.platform:
        libphcpack = LOCATION + "/libPHCpack.dylib"
        phcpack = ctypes.CDLL(libphcpack)
        return phcpack._ada_use_c2phc
    if 'win' in sys.platform:
        libphcpack = LOCATION + "/libPHCpack.dll"
        phcpack = ctypes.WinDLL(libphcpack, winmode=0)
        return phcpack._ada_use_c2phc
    print('The platform', sys.platform, 'is not supported.')
    return None

def get_phcfun(vrblvl=0):
    """
    Returns phcpy.phc, or if that not works,
    returns get_phcfun_fromlib()
    """
    if vrblvl > 0:
        print('in get_phcfun ...')
    try:
        import phcpy
        return phcpy.phc
    except:
        return get_phcfun_fromlib()

def int4a2str(data, vrblvl=0):
    """
    Given in data is an integer array stored as a ctypes string buffer,
    returns the string which represent the data.
    The '4a' refers to the 4-byte (or 32-bit) integer array.
    If vrblvl > 0, then results of intermediate steps are printed,
    otherwise, the function remains silent.
    """
    if vrblvl > 0:
        print('-> int4a2str, data is', data)
    szd = sizeof(data)
    if vrblvl > 0:
        print('-> int4a2str, size of the data is', szd)
    dim = szd//4
    if vrblvl > 0:
        print('-> int4a2str, number of characters :', dim)
    result = ""
    for k in range(dim):
        result = result + data[4*k].decode()
    if vrblvl > 0:
        print('-> int4a2str returns', result)
    return result

def str2int4a(data, vrblvl=0):
    """
    Given in data is a string, stored as a Python string,
    returns the 32-bit integer array representation,
    stored as a ctypes string buffer.
    If vrblvl > 0, then results of intermediate steps are printed,
    otherwise, the function remains silent.
    """
    if vrblvl > 0:
        print('-> str2int4a, data is', data)
    dim = len(data)
    if vrblvl > 0:
        print('-> str2int4a, size of the data is', dim)
    szd = 4*dim
    result = create_string_buffer(b"", szd)
    for k in range(dim):
        result[4*k] = data[k].encode()
    if vrblvl > 0:
        print('-> str2int4a returns', result)
    return result

def int4a2nbr(data, vrblvl=0):
    """
    Given in data is a Python list of integers,
    returns the encoding in a ctypes string buffer,
    for conversion into an 32-bit integer array.
    If vrblvl > 0, then results of intermediate steps are shown,
    otherwise the function remains silent.
    """
    if vrblvl > 0:
        print('-> int4a2nbr, data :', data)
    dim = len(data)
    szd = 4*dim
    if vrblvl > 0:
        print('-> int4a2nbr size of result :', szd)
    result = create_string_buffer(b"", szd)
    for k in range(dim):
        if data[k] < 256:
            result[4*k] = data[k] # no encode() because plain integer
        elif data[k] < 65536:
            result[4*k+1], result[4*k] = divmod(data[k], 256)
        elif data[k] < 16777216:
            result[4*k+2], rest = divmod(data[k], 65536)
            result[4*k+1], result[4*k] = divmod(rest, 256)
        else:
            result[4*k+3], rest = divmod(data[k], 16777216)
            result[4*k+2], rest = divmod(rest, 65536)
            result[4*k+1], result[4*k] = divmod(rest, 256)
    if vrblvl > 0:
        print('-> int4a2nbr return type :', type(result))
        print('-> int4a2nbr return value :', nbr2int4a(result))
    return result

def nbr2int4a(data, vrblvl=0):
    """
    Given in data is a 32-bit integer array,
    in a ctypes string buffer.
    Returns the list of python integers.
    """
    if vrblvl > 0:
        print('-> nbr2int4a, data type :', type(data))
    fmt = str(len(data)//4) + 'I'
    if vrblvl > 0:
        print('format :', fmt)
    result = list(unpack(fmt, data))
    if vrblvl > 0:
        print('-> nbr2int4a returns', result)
    return result

def version_string(vrblvl=0):
    """
    Returns the version string of PHCpack.
    If vrblvl > 0, then the conversions between strings and integer arrays
    are verified via the ctypes string buffer types.
    """
    if vrblvl > 0:
        print('in version_string ...')
    phc = get_phcfun(vrblvl)
    aaa = pointer(c_int32(0))
    name = create_string_buffer(b"", 30*4)
    ccc = pointer(c_double(0.0))
    if vrblvl > 0:
        retval = phc(999, aaa, name, ccc, 2)
    else:
        retval = phc(999, aaa, name, ccc, 0)
    if vrblvl > 0:
        print('return value :', retval)
        print('a :', aaa[0])
        print('sizeof(name) :', sizeof(name))
    result = int4a2str(name, vrblvl=vrblvl-1)
    if vrblvl > 0:
        print(result)
        print('version checks conversions ...')
        int4aresult = str2int4a(result, vrblvl=vrblvl-1)
        strres = int4a2str(int4aresult, vrblvl=vrblvl-1)
        print(strres)
    return result

def test_byte_strings(vrblvl=0):
    """
    Tests the conversion of a string into a string buffer
    and then backwards.
    """
    if vrblvl > 0:
        print('in test_byte_strings ...')
    greeting = 'Hello World!'
    if vrblvl > 0:
        print(greeting)
    coded = str2int4a(greeting, vrblvl-1)
    decoded = int4a2str(coded, vrblvl-1)
    if vrblvl > 0:
        print(decoded)
    if greeting == decoded:
        if vrblvl > 0:
            print('Test passed.')
        return 0
    if vrblvl > 0:
        print('Test failed!')
    return 1

def test_integer_encodings(vrblvl=0):
    """
    Tests the encoding of a list of integers as a ctypes string buffer.
    """
    if vrblvl > 0:
        print('in test_integer_encodings ...')
    first = [12, 1033, 129129, 20123543]
    print('L =', first)
    bfirst = int4a2nbr(first, vrblvl)
    second = nbr2int4a(bfirst, vrblvl)
    print('K =', second)
    if first == second:
        if vrblvl > 0:
            print('Test passed.')
        return 0
    if vrblvl > 0:
        print('Test failed!')
    return 1

def main():
    """
    Prints the version string and runs two tests.
    """
    lvl = 1
    print(version_string(lvl))
    fail = test_byte_strings(lvl)
    fail = fail + test_integer_encodings(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=="__main__":
    main()



n = [  16,    32,    48,     64,     80,     96,    112,    128,     144,     160,     176,     192,     208,     224,     240]
c = [2.01, 14.61, 47.80, 112.60, 217.52, 373.06, 589.35, 876.11, 1243.26, 1701.57, 2260.07, 2932.15, 3722.77, 4641.71, 5703.77]
g = [4.11,  6.52, 11.11,  15.38,  22.89,  30.43,  40.82,  49.10,   67.41,   80.42,   99.94,  116.90,  149.45,  172.30,  211.30]
for k in range(0,len(n)):
   print n[k], c[k], g[k], c[k]/g[k]

# the original times are below:
# [jan@dezon MGS]$ python run_mgs_timings.py
# running the modified Gram-Schmidt method
# type precision level (d, dd, or qd) : d
# give start dimension (>= 2) : 16
# give stop dimension (<= 254) : 255
# give the step : 16
# give frequency per dimension (>= 1) : 10000
# give the name of the output file : run_mgs_d_times
# running for dimension 16 and frequency 10000 on CPU
# ['real 2.01', 'user 1.99', 'sys 0.01']
# running for dimension 16 and frequency 10000 on GPU
# ['real 4.11', 'user 1.04', 'sys 1.89']
# running for dimension 32 and frequency 10000 on CPU
# ['real 14.61', 'user 14.55', 'sys 0.05']
# running for dimension 32 and frequency 10000 on GPU
# ['real 6.52', 'user 2.34', 'sys 2.94']
# running for dimension 48 and frequency 10000 on CPU
# ['real 47.80', 'user 47.61', 'sys 0.15']
# running for dimension 48 and frequency 10000 on GPU
# ['real 11.11', 'user 4.10', 'sys 5.65']
# running for dimension 64 and frequency 10000 on CPU
# ['real 112.60', 'user 112.29', 'sys 0.21']
# running for dimension 64 and frequency 10000 on GPU
# ['real 15.38', 'user 6.48', 'sys 7.17']
# running for dimension 80 and frequency 10000 on CPU
# ['real 217.52', 'user 216.96', 'sys 0.33']
# running for dimension 80 and frequency 10000 on GPU
# ['real 22.89', 'user 10.38', 'sys 11.38']
# running for dimension 96 and frequency 10000 on CPU
# ['real 373.06', 'user 372.20', 'sys 0.50']
# running for dimension 96 and frequency 10000 on GPU
# ['real 30.43', 'user 13.31', 'sys 15.21']
# running for dimension 112 and frequency 10000 on CPU
# ['real 589.35', 'user 588.07', 'sys 0.75']
# running for dimension 112 and frequency 10000 on GPU
# ['real 40.82', 'user 18.59', 'sys 21.22']
# running for dimension 128 and frequency 10000 on CPU
# ['real 876.11', 'user 874.32', 'sys 0.99']
# running for dimension 128 and frequency 10000 on GPU
# ['real 49.10', 'user 22.46', 'sys 25.50']
# running for dimension 144 and frequency 10000 on CPU
# ['real 1243.26', 'user 1240.84', 'sys 1.22']
# running for dimension 144 and frequency 10000 on GPU
# ['real 67.41', 'user 30.40', 'sys 35.86']
# running for dimension 160 and frequency 10000 on CPU
# ['real 1701.57', 'user 1698.37', 'sys 1.58']
# running for dimension 160 and frequency 10000 on GPU
# ['real 80.42', 'user 35.74', 'sys 43.11']
# running for dimension 176 and frequency 10000 on CPU
# ['real 2260.07', 'user 2256.19', 'sys 1.86']
# running for dimension 176 and frequency 10000 on GPU
# ['real 99.94', 'user 44.00', 'sys 54.77']
# running for dimension 192 and frequency 10000 on CPU
# ['real 2932.15', 'user 2926.93', 'sys 2.50']
# running for dimension 192 and frequency 10000 on GPU
# ['real 116.90', 'user 51.93', 'sys 63.43']
# running for dimension 208 and frequency 10000 on CPU
# ['real 3722.77', 'user 3716.35', 'sys 2.94']
# running for dimension 208 and frequency 10000 on GPU
# ['real 149.45', 'user 66.72', 'sys 81.44']
# running for dimension 224 and frequency 10000 on CPU
# ['real 4641.71', 'user 4634.09', 'sys 3.33']
# running for dimension 224 and frequency 10000 on GPU
# ['real 172.30', 'user 75.59', 'sys 94.71']
# running for dimension 240 and frequency 10000 on CPU
# ['real 5703.77', 'user 5694.41', 'sys 3.96']
# running for dimension 240 and frequency 10000 on GPU
# ['real 211.30', 'user 92.85', 'sys 117.15']



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# https://www.sphinx-doc.org/en/master/usage/configuration.html

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
import shlex
sys.path[:0] = [os.path.abspath('../../phcpy'), os.path.abspath('../..')]

# -- Project information -----------------------------------------------------

project = 'phcpy'
copyright = u'2016-2024, Jan Verschelde'
author = 'Jan Verschelde'

# The full version, including alpha/beta/rc tags
release = '1.1.4'

# -- General configuration ---------------------------------------------------

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom
# ones.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.mathjax',
]

# Add any paths that contain templates here, relative to this directory.
templates_path = ['_templates']

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = []

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
#
# html_theme = 'alabaster'
html_theme = 'classic'

# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
html_static_path = ['_static']

# -- Options for LaTeX output ---------------------------------------------

latex_elements = {
# The paper size ('letterpaper' or 'a4paper').
#'papersize': 'letterpaper',

# The font size ('10pt', '11pt' or '12pt').
#'pointsize': '10pt',

# Additional stuff for the LaTeX preamble.
'preamble': '\\usepackage{amssymb}',

# Latex figure (float) alignment
'figure_align': 'htbp',

# chapter headings
'fncychap': '\\usepackage[Sonny]{fncychap}',
}

# Grouping the document tree into LaTeX files. List of tuples
# (source start file, target name, title,
#  author, documentclass [howto, manual, or own class]).
latex_documents = [
  ('index', 'phcpy.tex', 'phcpy Documentation',
   'Jan Verschelde', 'manual'),
]

# The name of an image file (relative to this directory) to place at the top of
# the title page.
#latex_logo = None

# For "manual" documents, if this is true, then toplevel headings are parts,
# not chapters.
#latex_use_parts = False

# If true, show page references after internal links.
#latex_show_pagerefs = False

# If true, show URL addresses after external links.
#latex_show_urls = False

# Documents to append as an appendix to all manuals.
#latex_appendices = []

# If false, no module index is generated.
#latex_domain_indices = True

# numbering of figures
numfig = True




"""
Exports functions on solutions.
"""
from math import log10, floor
from ast import literal_eval
from ctypes import c_int32, c_double, pointer, create_string_buffer
from phcpy.version import get_phcfun, int4a2nbr, int4a2str, str2int4a

def diagnostics(sol, vrblvl=0):
    r"""
    Extracts the diagnostics (err, rco, res)
    from the PHCpack string solution in *sol* and
    returns a triplet of three floats.
    """
    if vrblvl > 0:
        print('in diagnostics, sol :')
        print(sol)
    banner = sol.split("==")
    data = banner[1]
    diag = data.split("=")
    str_err = diag[0].split(':')
    str_rco = diag[1].split(':')
    str_res = diag[2].split(':')
    val_err = literal_eval(str_err[1].lstrip())
    val_rco = literal_eval(str_rco[1].lstrip())
    val_res = literal_eval(str_res[1].lstrip())
    # print 'err =', val_err, 'rco =', val_rco, 'res =', val_res
    return (val_err, val_rco, val_res)

def map_double(freqtab, nbr, vrblvl=0):
    """
    On input in freqtab is a list of integer numbers and nbr is a double.
    The list freqtab represents a frequency table of magnitudes.
    The magnitude of the double nbr is mapped into the frequency table.
    The counter in freqtab that will be updated is at position
    floor(-log10(nbr)) 
    """
    if vrblvl > 0:
        print('in map_double, nbr :', nbr)
        print('freqtab :', freqtab)
    if nbr > 1.0:
        freqtab[0] = freqtab[0] + 1
    else:
        tol = 10.0**(-len(freqtab)+1)
        if nbr < tol:
            freqtab[len(freqtab)-1] = freqtab[len(freqtab)-1] + 1
        else:
            idx = floor(-log10(nbr))
            if idx < 0:
                freqtab[0] = freqtab[0] + 1
            elif idx >= len(freqtab):
                freqtab[len(freqtab)-1] = freqtab[len(freqtab)-1] + 1
            else:
                freqtab[idx] = freqtab[idx] + 1

def condition_tables(sols, vrblvl=0):
    """
    The input in sols is a list of PHCpack string solutions.
    A condition table is triplet of three frequency tables,
    computed from the diagnostics (err, rco, res) of each solution.
    The i-th entry in each frequency table counts the number of
    doubles x which floor(-log10(x)) mapped to the index i.
    Small numbers are mapped to the right of the table,
    large numbers are mapped to the left of the table.
    """
    if vrblvl > 0:
        print('in condition_tables, sols :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    errtab = [0 for _ in range(16)]
    rcotab = [0 for _ in range(16)]
    restab = [0 for _ in range(16)]
    for sol in sols:
        (err, rco, res) = diagnostics(sol)
        map_double(errtab, err)
        map_double(rcotab, rco)
        map_double(restab, res)
    return (errtab, rcotab, restab)

def str2complex(scn, vrblvl=0):
    r"""
    The string *scn* contains a complex number,
    the real and imaginary part separated by spaces.
    On return is the Python complex number.
    """
    if vrblvl > 0:
        print('in str2complex, scn :', scn)
    stripped = scn.strip()
    realimag = stripped.split(' ')
    realpart = realimag[0].replace('E', 'e')
    imagpart = realimag[len(realimag)-1].replace('E', 'e')
    return complex(literal_eval(realpart), literal_eval(imagpart))

def string_complex(scn, vrblvl=0):
    r"""
    The string *scn* contains a complex number,
    the real and imaginary part separated by spaces.
    On return is the string representation of a complex number,
    in Python format.  The use of this function is for when
    the coordinates are calculated in higher precision.
    """
    if vrblvl > 0:
        print('in string_complex, scn :', scn)
    stripped = scn.strip()
    realimag = stripped.split(' ')
    realpart = realimag[0].replace('E', 'e')
    imagpart = realimag[len(realimag)-1].replace('E', 'e')
    if imagpart[0] == '-':
        result = '(' + realpart + imagpart + 'j)'
    else:
        result = '(' + realpart + '+' + imagpart + 'j)'
    return result

def coordinates(sol, vrblvl=0):
    r"""
    Returns the coordinates of the solution
    in the PHCpack solution string *sol*,
    as a tuple of two lists: (names, values).
    The list names contains the strings of the variable names.
    The list values contains the complex values for the
    coordinates of the solution.  The entries in the list
    names correspond to the entries in the list values.
    """
    if vrblvl > 0:
        print('in coordinates, sol :')
        print(sol)
    banner = sol.split("==")
    data = banner[0]
    nums = data.split("the solution for t :")
    firstnums = nums[1]
    lines = firstnums.split('\n')
    vard = []
    vals = []
    for line in lines:
        if line != '':
            xval = line.split(':')
            vard.append(xval[0].strip())
            vals.append(str2complex(xval[1]))
    return (vard, vals)

def string_coordinates(sol, vrblvl=0):
    r"""
    Returns the coordinates of the solution
    in the PHCpack solution string *sol*,
    as a tuple of two lists: (names, values).
    For each name in names there is a value in values.
    The list names contains the strings of the variable names.
    The list values contains the values of the coordinates,
    represented as strings.  This function is useful for
    when the coordinates are computed in higher precision.
    """
    if vrblvl > 0:
        print('in string_coordinates, sol:')
        print(sol)
    banner = sol.split("==")
    data = banner[0]
    nums = data.split("the solution for t :")
    firstnums = nums[1]
    lines = firstnums.split('\n')
    vard = []
    vals = []
    for line in lines:
        if line != '':
            xval = line.split(':')
            vard.append(xval[0].strip())
            vals.append(string_complex(xval[1]))
    return (vard, vals)

def endmultiplicity(sol, vrblvl=0):
    r"""
    Returns the value of t at the end
    and the multiplicity as (t,m)
    for the PHCpack solution string *sol*.
    """
    if vrblvl > 0:
        print('in endmultiplicity, sol :')
        print(sol)
    data = sol.split("the solution for t :")
    tstr = data[0]
    line = tstr.split('\n')
    tstr = line[0].split(':')
    tval = str2complex(tstr[1])
    mstr = line[1].split(':')
    mval = literal_eval(mstr[1].lstrip())
    return (tval, mval)

def strsol2dict(sol, precision='d', vrblvl=0):
    r"""
    Converts the solution in the string *sol*
    into a dictionary format.
    By default, the precision of the coordinates is assumed
    to be double float ('d' on input).
    If the precision is not 'd', then the coordinates of the solution
    are returned as Python complex number string representations.
    """
    if vrblvl > 0:
        print('in strsol2dict, precision :', precision, 'sol :')
        print(sol)
    result = {}
    (tval, mult) = endmultiplicity(sol)
    result['t'] = tval
    result['m'] = mult
    (err, rco, res) = diagnostics(sol)
    result['err'] = err
    result['rco'] = rco
    result['res'] = res
    if precision == 'd':
        (var, val) = coordinates(sol)
    else:
        (var, val) = string_coordinates(sol)
    for idx, name in enumerate(var):
        result[name] = val[idx]
    return result

def formdictlist(sols, precision='d', vrblvl=0):
    r"""
    Given in *sols* is a list of strings.
    Each string in *sols* represents a solution,
    in the PHCpack format.
    On return is the list of dictionaries.
    Each dictionary in the list of return
    stores each solution of the list *sols*
    in the dictionary format.
    By default, the precision of the coordinates is assumed
    to be double float ('d' on input).
    If the precision is not 'd', then the coordinates of the solution
    are returned as Python complex number string representations.
    """
    if vrblvl > 0:
        print('in formdictlist, precision :', precision)
        print('sols :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    return [strsol2dict(sol, precision, vrblvl) for sol in sols]

def variables(dsol, vrblvl=0):
    r"""
    Given the dictionary format of a solution in *dsol*,
    returns the list of variables.
    """
    if vrblvl > 0:
        print('in veriables, dsol :')
        print(dsol)
    solkeys = list(dsol.keys())
    solkeys.remove('t')
    solkeys.remove('m')
    solkeys.remove('err')
    solkeys.remove('rco')
    solkeys.remove('res')
    return solkeys

def numerals(dsol, vrblvl=0):
    r"""
    Given the dictionary format of a solution *dsol*,
    returns the list of numeric values of the variables in the solution.
    """
    if vrblvl > 0:
        print('in numerals, dsol :')
        print(dsol)
    names = variables(dsol, vrblvl)
    return [dsol[name] for name in names]

def evaluate_polynomial(pol, dsol, vrblvl=0):
    r"""
    Evaluates the polynomial *pol* at the solution
    dictionary *dsol* by string substitution.
    """
    if vrblvl > 0:
        print('in evaluate_polynomial, pol :')
        print(pol)
        print('dsol :')
        print(dsol)
    j = complex(0, 1) # needed for eval(result) in test()
    varsd = variables(dsol, vrblvl)
    rpol = pol
    rpol = rpol.replace('i', 'j')
    rpol = rpol.replace('E', 'e')
    rpol = rpol.replace('^', '**')
    for varname in varsd:
        if isinstance(dsol[varname], complex):
            xre = f"{dsol[varname].real:+.17f}"
            xim = f"{dsol[varname].imag:+.17f}"
            value = '(' + xre + xim + 'j)'
        else:
            value = dsol[varname]
        rpol = rpol.replace(varname, value)
    result = rpol[:-1]
    return eval(result)

def evaluate(pols, dsol, vrblvl=0):
    r"""
    Evaluates a list of polynomials given as string in *pols*
    at the solution in dictionary format in *dsol*.
    """
    if vrblvl > 0:
        print('in evaluate, pols :')
        for pol in pols:
            print(pol)
        print('dsol :')
        print(dsol)
    result = []
    for pol in pols:
        result.append(evaluate_polynomial(pol, dsol, vrblvl))
    return result

def verify(pols, sols, vrblvl=0):
    r"""
    Verifies whether the solutions in *sols*
    satisfy the polynomials of the system in *pols*.
    Returns the sum of the absolute values of the 
    residuals in all polynomials.
    """
    if vrblvl > 0:
        print('in verify, pols :')
        for pol in pols:
            print(pol)
        print('sols :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    dictsols = [strsol2dict(sol, vrblvl=vrblvl) for sol in sols]
    checksum = 0
    for (idx, sol) in enumerate(dictsols):
        eva = evaluate(pols, sol, vrblvl-1)
        sumeval = sum(abs(nbr) for nbr in eva)
        if vrblvl > 0:
            print('sum at solution', idx, ':', sumeval)
        checksum = checksum + sumeval
    if vrblvl > 0:
        print('the total check sum :', checksum)
    return checksum.real

def make_solution(names, values, \
    err=0.0, rco=1.0, res=0.0, tval=0, multiplicity=1, vrblvl=0):
    r"""
    Makes the string representation in PHCpack format
    with in *names* a list of strings for the variables names
    and in *values* a list of (complex) values for the coordinates.
    For example:

    s = make_solution(['x','y'],[(1+2j), 3])

    returns the string s to represent the solution with
    coordinates (1+2j) and 3 for the variables x and y.
    The imaginary unit is the Python j instead of i.
    Other arguments for this function are

    1. *err* is the magnitude of an update, or the forward error,

    2. *rco* is the estimate for the inverse condition number,

    3. *res* is the value of the residual, or backward error,

    4. *tval* is the value for the continuation parameter t,

    5. *multiplicity* is the multiplicity of the solution.

    For those above arguments, default values are provided.
    Applying the function coordinates on the result of
    make_solution returns the tuple of arguments given
    on input to **make_solution()**.
    """
    if vrblvl > 0:
        print('in make_solution, names :', names)
        print('values :', values)
    if isinstance(tval, complex):
        (tre, tim) = (tval.real, tval.imag)
    elif isinstance(tval, float):
        (tre, tim) = (tval, 0.0)
    elif isinstance(tval, int):
        (tre, tim) = (tval, 0.0)
    else:
        print('wrong type for the value of the continuation parameter t')
        return ""
    result = f"t : {tre:.15E} {tim:.15E}\n"
    mstr = f"m : {multiplicity}\n"
    result = result + mstr
    result = result + 'the solution for t :\n'
    for idx, name in enumerate(names):
        result = result + ' ' + name + ' : '
        if isinstance(values[idx], complex):
            c_re = f"{values[idx].real:.15E}"
            c_im = f"{values[idx].imag:.15E}"
            result = result + c_re + '  ' + c_im + '\n'
        elif isinstance(values[idx], float):
            flt = f"{values[idx]:.15E}"
            result = result + flt + '  ' + '0.0' + '\n'
        elif isinstance(values[idx], int):
            i = f"{values[idx]:.15E}"
            result = result + i + '  ' + '0.0' + '\n'
        else:
            print('wrong type for coordinate value')
            return result
    lastline = f"== err : {err:.3E} = rco : {rco:.3E} = res : {res:.3E} ="
    result = result + lastline
    return result

def is_real(sol, tol, vrblvl=0):
    r"""
    Returns True if the solution in *sol* is real with respect
    to the given tolerance *tol*: if the absolute value of the imaginary
    part of all coordinates are less than *tol*.
    """
    if vrblvl > 0:
        print('in is_real, tol :', tol)
        print('sol :')
        print(sol)
    (_, vals) = coordinates(sol, vrblvl)
    for value in vals:
        if abs(value.imag) > tol:
            return False
    return True

def filter_real(sols, tol, oper, vrblvl=0):
    r"""
    Filters the real solutions in *sols*.
    The input parameters are

    1. *sols* is a list of solution strings in PHCpack format,

    2. *tol* is the tolerance on the absolute value of the
       imaginary parts of the coordinates of the solution.

    3. *oper* is either 'select' or 'remove'

       if *oper* == 'select' then solutions that are considered real
       are selected and in the list on return,

       if *oper* == 'remove' then solutions that are considered real
       are in the list on return.
    """
    if vrblvl > 0:
        print('in filter_real, tol :', tol, ', oper :', oper)
        print('sols :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    result = []
    for sol in sols:
        isreal = is_real(sol, tol, vrblvl)
        if oper == 'select':
            if isreal:
                result.append(sol)
        if oper == 'remove':
            if not isreal:
                result.append(sol)
    return result

def filter_regular(sols, tol, oper, vrblvl=0):
    r"""
    Filters solutions in *sols* for the estimate of
    the inverse of the condition number.
    The input parameters are

    1. *sols* is a list of solution strings in PHCpack format,

    2. *tol* is the tolerance on the value for the estimate rco
       for the inverse of the condition number to decide whether
       a solution is singular (if rco < *tol*) or not.

    3. *oper* is either 'select' or 'remove'

       if *oper* == 'select' then solutions with value rco > *tol*
       are selected and in the list on return,

       if *oper* == 'remove' then solutions with value rco <= *tol*
       are in the list on return.
    """
    if vrblvl > 0:
        print('in filter_regular, tol :', tol, ', oper :', oper)
        print('sols :')
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    result = []
    for sol in sols:
        rco = diagnostics(sol, vrblvl)[1]
        if oper == 'select':
            if rco > tol:
                result.append(sol)
        if oper == 'remove':
            if rco <= tol:
                result.append(sol)
    return result

def filter_zero_coordinates(sols, varname, tol, oper, vrblvl=0):
    r"""
    Filters the solutions in *sols* for variables
    that have a value less than the tolerance.
    The input parameters are

    1. *sols* is a list of solution strings in PHCpack format,

    2. *varname* is a string with the name of the variable,

    3. *tol* is the tolerance to decide whether a complex
       number equals zero or not, and

    4. *oper* is either 'select' or 'remove'

       if *oper* == 'select' then solutions with value for the
       variable v that is less than *tol* are selected and
       in the list on return,

       if *oper* == 'remove' then solutions with value for the
       variable v that is less than *tol* are removed and
       not in the list on return.
    """
    if vrblvl > 0:
        print('in filter_zero_coordinates, tol :', tol)
        print(', varname :', varname, end='')
        print(', oper :', oper)
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    result = []
    for sol in sols:
        dsol = strsol2dict(sol, vrblvl=vrblvl)
        if oper == 'select':
            if abs(dsol[varname]) < tol:
                result.append(sol)
        if oper == 'remove':
            if abs(dsol[varname]) >= tol:
                result.append(sol)
    return result

def is_vanishing(sol, tol, vrblvl=0):
    r"""
    Given in *sol* is a solution string and
    *tol* is the tolerance on the residual.
    Returns True if the residual of *sol*
    is less than or equal to *tol*.
    Returns False otherwise.
    """
    if vrblvl > 0:
        print('in is_vanishing, tol :', tol)
        print('sol :')
        print(sol)
    dgn = diagnostics(sol, vrblvl)
    return dgn[2] <= tol

def filter_vanishing(sols, tol, vrblvl=0):
    r"""
    Returns the list of solutions in *sols* that have a residual
    less than or equal to the given tolerance in *tol*.
    """
    if vrblvl > 0:
        print('in filter_vanishing, tol :', tol)
        for (idx, sol) in enumerate(sols):
            print('Solution', idx+1, ':')
            print(sol)
    result = []
    for sol in sols:
        if is_vanishing(sol, tol, vrblvl):
            result.append(sol)
    return result

def append_double_solution_string(nvr, sol, vrblvl=0):
    """
    Appends the string in sol to the solutions in double precision,
    where the number of variables equals nvr.
    """
    if vrblvl > 0:
        print('-> append_double_solution_string, nvr =', nvr)
        print('The solution string :')
        print(sol)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([nvr, len(sol)], vrblvl=vrblvl-1)
    bsol = str2int4a(sol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> append_double_solution_string calls phc', end='')
    retval = phc(208, apars, bsol, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def append_double_double_solution_string(nvr, sol, vrblvl=0):
    """
    Appends the string in sol to the solutions in double double precision,
    where the number of variables equals nvr.
    """
    if vrblvl > 0:
        print('-> append_double_double_solution_string, nvr =', nvr)
        print('The solution string :')
        print(sol)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([nvr, len(sol)], vrblvl=vrblvl-1)
    bsol = str2int4a(sol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> append_double_double_solution_string calls phc', end='')
    retval = phc(378, apars, bsol, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def append_quad_double_solution_string(nvr, sol, vrblvl=0):
    """
    Appends the string in sol to the solutions in quad double precision,
    where the number of variables equals nvr.
    """
    if vrblvl > 0:
        print('-> append_quad_double_solution_string, nvr =', nvr)
        print('The solution string :')
        print(sol)
    phc = get_phcfun(vrblvl-1)
    apars = int4a2nbr([nvr, len(sol)], vrblvl=vrblvl-1)
    bsol = str2int4a(sol, vrblvl=vrblvl-1)
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> append_quad_double_solution_string calls phc', end='')
    retval = phc(428, apars, bsol, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_solutions(vrblvl=0):
    """
    Clears the solutions defined in double precision.
    """
    if vrblvl > 0:
        print('in clear_double_solutions ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_solutions calls phc', end='')
    retval = phc(37, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_double_double_solutions(vrblvl=0):
    """
    Clears the solutions defined in double double precision.
    """
    if vrblvl > 0:
        print('in clear_double_double_solutions ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_double_double_solutions calls phc', end='')
    retval = phc(347, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def clear_quad_double_solutions(vrblvl=0):
    """
    Clears the solutions defined in quad double precision.
    """
    if vrblvl > 0:
        print('in clear_quad_double_solutions ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> clear_quad_double_solutions calls phc', end='')
    retval = phc(397, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def set_double_solutions(nvr, sols, vrblvl=0):
    """
    Sets the solutions in double precision, with the strings in sols,
    where the number of variables equals nvr.
    """
    if vrblvl > 0:
        print('-> set_double_solutions, nvr :', nvr)
    clear_double_solutions(vrblvl)
    fail = 0
    for (ind, sol) in enumerate(sols):
        fail = append_double_solution_string(nvr, sol, vrblvl)
        if fail != 0:
            print('Solution at position', ind, 'is not appended.')
    return fail

def set_double_double_solutions(nvr, sols, vrblvl=0):
    """
    Sets the solutions in double double precision, with the strings in sols,
    where the number of variables equals nvr.
    """
    if vrblvl > 0:
        print('in set_double_double_solutions, nvr :', nvr)
    clear_double_double_solutions(vrblvl)
    fail = 0
    for (ind, sol) in enumerate(sols):
        fail = append_double_double_solution_string(nvr, sol, vrblvl)
        if fail != 0:
            print('Solution at position', ind, 'is not appended.')
    return fail

def set_quad_double_solutions(nvr, sols, vrblvl=0):
    """
    Sets the solutions in quad double precision, with the strings in sols,
    where the number of variables equals nvr.
    """
    if vrblvl > 0:
        print('in set_quad_double_solutions, nvr :', nvr)
    clear_quad_double_solutions(vrblvl)
    fail = 0
    for (ind, sol) in enumerate(sols):
        fail = append_quad_double_solution_string(nvr, sol, vrblvl)
        if fail != 0:
            print('Solution at position', ind, 'is not appended.')
    return fail

def number_double_solutions(vrblvl=0):
    """
    Returns the number of solutions in double precision.
    The vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in number_double_solutions ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> number_double_solutions calls phc', end='')
    retval = phc(32, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return bbb[0]

def number_double_double_solutions(vrblvl=0):
    """
    Returns the number of solutions in double double precision.
    The vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in number_double_double_solutions ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> number_double_double_solutions calls phc', end='')
    retval = phc(342, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return bbb[0]

def number_quad_double_solutions(vrblvl=0):
    """
    Returns the number of solutions in quad double precision.
    The vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in number_quad_double_solutions ...')
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> number_quad_double_solutions calls phc', end='')
    retval = phc(392, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return bbb[0]

def get_next_double_solution(idx, vrblvl=0):
    """
    Returns the string representation of the next solution
    in double precision, at the index idx.
    The vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in get_next_double_solution, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(idx)) # at the given index
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_next_double_solution calls phc', end='')
    retval = phc(525, aaa, bbb, ccc, vrb)
    size = bbb[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> get_next_double_solution, size :', size)
    soldata = create_string_buffer(b"", 4*size)
    if vrblvl > 0:
        print('-> get_next_double_solution calls phc', end='')
    retval = phc(533, bbb, soldata, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = int4a2str(soldata, vrblvl=vrblvl-1)
    return result

def get_next_double_double_solution(idx, vrblvl=0):
    """
    Returns the string representation of the next solution
    in double double precision, at the index idx.
    The vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in get_next_double_double_solution, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(idx)) # at the given index
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_next_double_double_solution calls phc', end='')
    retval = phc(526, aaa, bbb, ccc, vrb)
    size = bbb[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> get_next_double_double_solution, size :', size)
    soldata = create_string_buffer(b"", 4*size)
    if vrblvl > 0:
        print('-> get_next_double_double_solution calls phc', end='')
    retval = phc(534, bbb, soldata, ccc, vrb)
    if vrblvl > 0:
        print(", return value :", retval)
    result = int4a2str(soldata, vrblvl=vrblvl-1)
    return result

def get_next_quad_double_solution(idx, vrblvl=0):
    """
    Returns the string representation of the next solution
    in quad double precision, at the index idx.
    The vrblvl is the verbose level.
    """
    if vrblvl > 0:
        print('in get_next_quad_double_solution, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(idx)) # at the given index
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> get_next_quad_double_solution calls phc', end='')
    retval = phc(527, aaa, bbb, ccc, vrb)
    size = bbb[0]
    if vrblvl > 0:
        print(', return value :', retval)
        print('-> get_next_quad_double_solution, size :', size)
    soldata = create_string_buffer(b"", 4*size)
    if vrblvl > 0:
        print('-> get_next_quad_double_solution calls phc', end='')
    retval = phc(535, bbb, soldata, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    result = int4a2str(soldata, vrblvl=vrblvl-1)
    return result

def move_double_solution_cursor(idx, vrblvl=0):
    """
    Moves the cursor to the next solution, following the index,
    in double precision.
    """
    if vrblvl > 0:
        print('in move_double_solution_cursor, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(idx)) # at the given index
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> move_double_solution_cursor calls phc', end='')
    retval = phc(454, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return aaa[0]

def move_double_double_solution_cursor(idx, vrblvl=0):
    """
    Moves the cursor to the next solution, following the index,
    in double double precision.
    """
    if vrblvl > 0:
        print('in move_double_double_solution_cursor, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(idx)) # at the given index
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> move_double_double_solution_cursor calls phc', end='')
    retval = phc(455, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return aaa[0]

def move_quad_double_solution_cursor(idx, vrblvl=0):
    """
    Moves the cursor to the next solution, following the index,
    in quad double precision.
    """
    if vrblvl > 0:
        print('in move_quad_double_solution_cursor, idx :', idx)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(idx)) # at the given index
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> move_quad_double_solution_cursor calls phc', end='')
    retval = phc(456, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return aaa[0]

def get_double_solutions(vrblvl=0):
    """
    Returns the solution strings in double precision.
    """
    if vrblvl > 0:
        print('in get_double_solutions ...')
    nbrsols = number_double_solutions(vrblvl)
    if vrblvl > 0:
        print('number of solutions retrieved :', nbrsols)
    result = []
    if nbrsols > 0:
        sol = get_next_double_solution(1, vrblvl)
        if vrblvl > 0:
            print('the first solution :\n', sol)
        result.append(sol)
        idx = 1
        for _ in range(1, nbrsols):
            idx = move_double_solution_cursor(idx, vrblvl)
            if vrblvl > 0:
                print('the next index :', idx)
            sol = get_next_double_solution(idx, vrblvl)
            if vrblvl > 0:
                print('the solution at index', idx, ':\n', sol)
            result.append(sol)
    return result

def get_double_double_solutions(vrblvl=0):
    """
    Returns the solution strings in double double precision.
    """
    if vrblvl > 0:
        print('in get_double_double_solutions ...')
    nbrsols = number_double_double_solutions(vrblvl)
    if vrblvl > 0:
        print('number of solutions retrieved :', nbrsols)
    result = []
    if nbrsols > 0:
        sol = get_next_double_double_solution(1, vrblvl)
        if vrblvl > 0:
            print('the first solution :\n', sol)
        result.append(sol)
        idx = 1
        for _ in range(1, nbrsols):
            idx = move_double_double_solution_cursor(idx, vrblvl)
            if vrblvl > 0:
                print('the next index :', idx)
            sol = get_next_double_double_solution(idx, vrblvl)
            if vrblvl > 0:
                print('the solution at index', idx, ':\n', sol)
            result.append(sol)
    return result

def get_quad_double_solutions(vrblvl=0):
    """
    Returns the solution strings in quad double precision.
    """
    if vrblvl > 0:
        print('in get_quad_double_solutions ...')
    nbrsols = number_quad_double_solutions(vrblvl)
    if vrblvl > 0:
        print('number of solutions retrieved :', nbrsols)
    result = []
    if nbrsols > 0:
        sol = get_next_quad_double_solution(1, vrblvl)
        if vrblvl > 0:
            print('the first solution :\n', sol)
        result.append(sol)
        idx = 1
        for _ in range(1, nbrsols):
            idx = move_quad_double_solution_cursor(idx, vrblvl)
            if vrblvl > 0:
                print('the next index :', idx)
            sol = get_next_quad_double_solution(idx, vrblvl)
            if vrblvl > 0:
                print('the solution at index', idx, ':\n', sol)
            result.append(sol)
    return result

def write_double_solutions(vrblvl=0):
    """
    Writes the solutions stored in double precision.
    """
    if vrblvl > 0:
        print('in write_double_solutions, vrblvl :', vrblvl)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> write_double_solutions calls phc', end='')
    retval = phc(31, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def write_double_double_solutions(vrblvl=0):
    """
    Writes the solutions stored in double double precision.
    """
    if vrblvl > 0:
        print('in write_double_double_solutions, vrblvl :', vrblvl)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> write_double_double_solutions calls phc', end='')
    retval = phc(341, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

def write_quad_double_solutions(vrblvl=0):
    """
    Writes the solutions stored in quad double precision.
    """
    if vrblvl > 0:
        print('in write_quad_double_solutions, vrblvl :', vrblvl)
    phc = get_phcfun(vrblvl-1)
    aaa = pointer(c_int32(0))
    bbb = pointer(c_int32(0))
    ccc = pointer(c_double(0.0))
    vrb = c_int32(vrblvl-1)
    if vrblvl > 0:
        print('-> write_quad_double_solutions calls phc', end='')
    retval = phc(391, aaa, bbb, ccc, vrb)
    if vrblvl > 0:
        print(', return value :', retval)
    return retval

class DoubleSolution():
    """
    Wraps the functions on solution strings.
    """
    def __init__(self, sol):
        """
        A solution is constructed from the
        string representation in PHCpack format,
        or otherwise from a dictionary with keys
        the names of the variables and values
        the complex numbers for the corresponding
        coordinates of the solution.
        """
        if isinstance(sol, str):
            self.dict = strsol2dict(sol)
        elif isinstance(sol, dict):
            self.dict = sol
            if 'err' not in self.dict:
                self.dict['err'] = 0.0
            if 'rco' not in self.dict:
                self.dict['rco'] = 1.0
            if 'res' not in self.dict:
                self.dict['res'] = 0.0
            if 'm' not in self.dict:
                self.dict['m'] = 1
            if 't' not in self.dict:
                self.dict['t'] = complex(0.0)
        else:
            print('Wrong argument type, provide string or dict.')

    def __str__(self):
        """
        Returns the string representation of a solution.
        """
        result = make_solution(variables(self.dict), numerals(self.dict), \
            err=self.dict['err'], rco=self.dict['rco'], res=self.dict['res'], \
            tval=self.dict['t'], multiplicity=self.dict['m'])
        return result

    def __repr__(self):
        """
        Defines the representation as the string representation.
        """
        return str(self)

    def coordinates(self):
        """
        Returns the values of the coordinates of the solution,
        as a tuple of variable names and corresponding values.
        """
        return (variables(self.dict), numerals(self.dict))

    def dictionary(self):
        """
        Returns the dictionary format of the solution.
        """
        return self.dict

    def variables(self):
        """
        Returns the variable names of the coordinates.
        """
        return variables(self.dict)

    def numerals(self):
        """
        Returns the numerical values of the coordinates.
        """
        return numerals(self.dict)

    def diagnostics(self):
        """
        Returns the numerical diagnostics.
        """
        return (self.dict['err'], self.dict['rco'], self.dict['res'])

    def multiplicity(self):
        """
        Returns the multiplicity.
        """
        return self.dict['m']

    def timevalue(self):
        """
        Returns the value of the continuation parameter.
        """
        return self.dict['t']

def test_double_functions(vrblvl=0):
    """
    Generates a random trinomial system,
    solves it, converts the solutions,
    and then sums the multiplicities.
    The verbose level is given by vrblvl.
    """
    if vrblvl > 0:
        print('in test_double_functions ...')
    pols = ['(x - 1)*(y - 1);', '(x + 1)*(y + 1);']
    names = ['x', 'y']
    sol1 = make_solution(names, [1, -1], vrblvl=vrblvl)
    sol2 = make_solution(names, [-1, 1], vrblvl=vrblvl)
    sols = [sol1, sol2]
    dsols = [strsol2dict(sol, vrblvl=vrblvl) for sol in sols]
    mult = 0
    s0d = strsol2dict(sols[0], vrblvl=vrblvl)
    if vrblvl > 0:
        print('variables :', variables(s0d))
        print(evaluate(pols, s0d, vrblvl))
    for sol in dsols:
        mult = mult + sol['m']
    if vrblvl > 0:
        print('sum of multiplicities :', mult)
        print('sum of values at the solutions :')
    errsum = 0
    for sol in dsols:
        if vrblvl > 0:
            print(sum(evaluate(pols, sol, vrblvl)))
        errsum = errsum + abs(sum(evaluate(pols, sol, vrblvl)))
    if vrblvl > 0:
        print('error sum :', errsum)
    return errsum > 1.0e-14

def test_double_solution_class(vrblvl=0):
    """
    Tests the methods in the class DoubleSolution.
    The verbose level is given by vrblvl.
    """
    mysol = DoubleSolution({'x': complex(1,2), 'y': complex(-7,0)})
    if vrblvl > 0:
        print('my solution :')
        print(mysol)
    names = ['x', 'y']
    sol1 = make_solution(names, [1, 1], err=0.01, rco= 1.2, res=0.01)
    sol2 = make_solution(names, [1, -1], err=0.1, rco= 1.1, res=0.03)
    sols = [sol1, sol2]
    sol = DoubleSolution(sols[0])
    if vrblvl > 0:
        print('the first solution :')
        print(sol)
        print('its coordinates :\n', sol.coordinates())
        print('its variable names :\n ', sol.variables())
        print('its numerical values :\n', sol.numerals())
        print('its diagnostics :\n', sol.diagnostics())
        print('its continuation parameter :', sol.timevalue())
        print('its multiplicity :', sol.multiplicity())
        print('the dictionary :\n', sol.dictionary())
    (errtab, rcotab, restab) = condition_tables(sols)
    if vrblvl > 0:
        print('The frequency tables for err, rco, and res:')
        print(errtab)
        print(rcotab)
        print(restab)
    return 0

def main():
    """
    Runs some tests on solutions.
    """
    lvl = 1
    fail = test_double_functions(lvl)
    fail = fail + test_double_solution_class(lvl)
    if fail == 0:
        print('=> All tests passed.')
    else:
        print('Number of failed tests :', fail)

if __name__=='__main__':
    main()



import cgi

from os import chdir, path 

from phc_client import PHC_Server
from phc_cookie import UpdateCookie, CookieUid, RemoveUid

def ProcessName(form):
   """
   Processes name of Email form.
   Returns True if error, else False.
   """
   error = 0
   usermail = ''
   userpwd = ''
   try:
      usermail = form['login'].value
   except KeyError:
      #print "Please enter your email"
      error = 1
   try:
      userpwd = form['passw'].value
   except KeyError:
      #print "Please enter your password"
      error = error + 2
   return error, usermail, userpwd

def AccessFolder(Folder):
   error = 0

   ## The account hasn't be activated
   if Folder == "None":
      error = 5
      return error
   elif len(Folder) != 40:
      error = 6
      return error
   Upath="../users/"+Folder
   Open_Folder = path.isdir(Upath)
   if not Open_Folder:
      PHC_Server(Folder,'3',Folder)
      error=7
   return error

def PHC_login(form,phcdb):
   import Cookie
   error = 0
   Folder = ''
   Name_First=''
   Status = None
   if form.has_key('Signout'):
      error = 10  # Error 10 is Sign out
      UpdateCookie(form, '', '', error)
   else:
      if form.has_key('phcaction'):
         error, usermail, userpwd = ProcessName(form)
         if not error:
            error, Folder, Name_First, Status= phcdb.check(usermail, userpwd)
            UpdateCookie(form, Folder, Name_First, error)
      else:
         Folder, Name_First = CookieUid()
         if Folder == '':
            error = 5 # haven't signin
         else:
            Status = phcdb.check_userstatus(Folder)
   if not error:
      error = AccessFolder(Folder)
   return error, Folder, Name_First, Status
   
def phc_login_simple():
   from sql_login import PHC_login
   from phc_sql import PHCSQL
   from phc_html import Print_User_Bar
   form = cgi.FieldStorage()
   # Processing cookie and user pass ### put this part into cookie_login.py
   phcdb = PHCSQL()
   error, Uid, Name_First, Status = PHC_login(form,phcdb)
   #print "Uid = %s</p>" % Uid
   if not error or error == 7: # error 7: folder not exist, recovery
      Print_User_Bar(Name_First, 1)
   else:      
      Print_User_Bar(Name_First, 0)
   phcdb.db.close()




#!/usr/bin/python

# Enter debug mode
from phc_config import phc_debug, style_about
if phc_debug:
   import cgitb; cgitb.enable()
   
import cgi
from phc_html import PrintHeader, Footer_Bar

from sql_login import phc_login_simple

def print_about():
   
   print """<div><p>This is a beta version of a web interface to PHCpack,
developed by Xiangcheng Yu.</p>

<p>Its current version exports the blackbox solver (phc -b) and
tracking paths defined by a homotopy in one parameter (phc -p).</p>

<p>The blackbox solver computes the mixed volume of the Newton polytopes
of the system, via an integrated version of Algorithm 846: MixedVol:
A software package for mixed volume computation, by T. Gao, T.Y. Li,
and M. Wu, published in ACM Trans. Math. Softw. 31(4):555-560, 2005.</p>
   
<div style="text-align:center;"><img src="../imag/Path.jpg" alt="PHC Web Interface" width="400"></div>
<p>The web interface logo results from running HomLab in Matlab:

The Numerical Solution of Systems of Polynomials Arising in Engineering and Science, by Andrew J. Sommese and Charles W. Wampler, II, World Scientific, 2005.</p>

<div>"""

def main():
   """
   About
   """
   # PHCpack Web Interface Header
   
   PrintHeader(style_about)
   
   phc_login_simple()
   
   print_about()
   
   Footer_Bar()
   
   print "</body></html>\n"

main()



"""
A Schubert problem on k-planes in n-space is defined by a sequence of m
brackets that specify at what dimensions the k-planes intersect m flags.
This script prompts the user for n and k and then generates 3 random
intersection conditions on k-planes in n-space.
Another possibility is to enumerate all possible triple conditions.
"""

from random import randint
from phcpy.schubert import resolve_schubert_conditions as resolve

def partition2bracket(n, k, p):
    """
    Given in p a list of k numbers representing a partition
    for conditions of a k-plane in n-space,
    returns the corresponding bracket notation for the conditions.
    """
    result = []
    for i in range(k):
        result.append(n - k + i + 1 - p[i])
    return result

def random_partition(n, k):
    """
    Returns a decreasing list of k numbers, entries of at most n-k,
    generated uniformly with the first number at least 1.
    """
    result = [randint(1, n-k-1)]
    for i in range(k-1):
        result.append(randint(0, result[i]))
    return result

def generate_all_partitions2(n, k, L, fun=None):
    """
    Prints the partition if len(L) == k (or calls fun if not None)
    and then continues the generation.
    The argument fun is a callback function, called with n, k, and L.
    """
    if(len(L) == k):
        if(fun == None):
            print L
        else:
            fun(n, k, L)
    else:
        lastval = L[len(L)-1]+1
        if(len(L) == k-1):
            if(lastval == n-k):
                lastval = lastval-1
        for j in range(0, lastval):
            generate_all_partitions2(n, k, L+[j], fun)

def generate_all_partitions(n, k, fun=None):
    """
    Prints all partitions for a k-plane in n-space,
    following the specifications of the above generate_partition(n, k).
    The last argument is a callback function, called with n, k,
    and the list that represents the generated partition.
    """
    for i in range(1, n-k):
        # print 'i =', i
        generate_all_partitions2(n, k, [i], fun)

def random_complement(n, k, p):
    """
    Returns an increasing list q of k numbers, with the restriction
    that p[i] + q[i] < n - k.  The last element is at least 1.
    """
    result = [randint(0, n-k-p[0]-1)]
    for i in range(k-1):
        result.append(randint(result[i],n-k-p[i+1]-1))
    if(result[k-1] == 0):
        result[k-1] = 1
    return result

def generate_all_complements2(n, k, p, L, fun=None):
    """
    Prints the complement if len(L) == k (or calls fun in not None)
    and then continues the generation.
    The argument fun is a callback function, called with n, k, p, and L.
    """
    if(len(L) == k):
        if(fun == None):
            print 'pair :', p, L
        else:
            fun(n, k, p, L)
    else:
        idx = len(L)-1
        firstval = L[idx]
        if(len(L) == k-1):
            if(firstval == 0):
                firstval = 1
        for j in range(firstval, n-k-p[idx+1]):
            generate_all_complements2(n, k, p, L+[j], fun)

def generate_all_complements(n, k, p, fun=None):
    """
    Prints all complements of the partition p for a k-plane in n-space,
    following the specifications of the above generate_complement(n, k, p).
    The callback function takes as arguments n, k, p, and the complement L.
    """
    for i in range(n-k-p[0]):
        # print 'i =', i
        generate_all_complements2(n, k, p, [i], fun)

def difference(n, k, p, q):
    """
    Returns the list of elements n - k - p[i] - q[i].
    """
    result = []
    for i in range(len(p)):
        result.append(n - k - p[i] - q[i])
    return result

def random_difference(n, k, p, q):
    """
    Returns a difference that has the same sum as
    the difference between p and q, but with a redistribution.
    """
    diff = difference(n, k, p, q)
    print 'the original difference :', diff
    rootcount = 0
    while(rootcount == 0):
        S = sum(diff)
        result = []
        for i in range(k-1):
            if(S > n-k):
                r = randint(0, n-k)
            else:
                r = randint(0, S)
            result.append(r)
            S = S - r
        result.append(S)
        result.sort(reverse=True)
        print 'original:', diff, 'redistributed:', result
        ind = 0
        while((ind < k-1) and (result[ind] > n-k)):
            d = result[ind] - n + k
            result[ind] = n-k
            result[ind+1] = result[ind+1] + d
            ind = ind + 1
        result.sort(reverse=True)
        q.sort(reverse=True)
        bp = partition2bracket(n, k, p)
        bq = partition2bracket(n, k, q)
        br = partition2bracket(n, k, result)
        rootcount = resolve(n, k, [bp, bq, br], False)
    return result

def generate_all_differences2(n, k, p, q, val, r, fun=None):
    """
    Recursive enumerator for all differences,
    as called by generate_all_differences.
    """
    if(len(r) == k):
        br = partition2bracket(n, k, r)
        rc = resolve(n, k, [p, q, br], False)
        if(rc > 2):
            if(fun != None):
                fun(p,q,br,rc)
            else:
                print 'brackets :', [p, q, br], 'root count :', rc
    elif(len(r) == k - 1):
        if(val <= r[k-2]):
            generate_all_differences2(n, k, p, q, 0, r+[val], fun)
    else:
        upper = min([val, n-k, r[len(r)-1]])
        lower = val/(k - len(r))
        for i in range(lower, upper+1):
            generate_all_differences2(n, k, p, q, val-i, r+[i], fun)

def generate_all_differences(n, k, p, q, fun=None):
    """
    Computes the difference between p and q, given as partitions,
    generates all possible redistributions of the conditions
    imposed by that difference.  Each time a new triplet with
    nonzero root count is generated, the brackets are printed.
    """
    diff = difference(n, k, p, q)
    S = sum(diff)
    lower = S/k # need to redistribute S into a partition
    upper = min([S, n-k])
    q.sort(reverse=True)
    bp = partition2bracket(n, k, p)
    bq = partition2bracket(n, k, q)
    for i in range(lower,upper+1):
        generate_all_differences2(n, k, bp, bq, S-i, [i], fun)

def check_sum(n, k, p, q, r):
    """
    Given three partitions in p, q, and r,
    checks whether the sum of their entries equals k*(n-k).
    """
    S = sum(p) + sum(q) + sum(r)
    c = k*(n-k)
    ps = ', %d*(%d-%d) = ' % (n, n, k)
    result = (S == c)
    print 'sum : ' + str(S) + ps + str(c) + ' ' + str(result)

def random_triplet(n, k):
    """
    Generates a triple intersection condition on k-planes in n-space,
    returned as three partitions.
    """
    p = random_partition(n, k)
    print 'the first random partition :', p
    q = random_complement(n, k, p)
    print 'the second random partition :', q
    print 'the second random partition :', q
    r = random_difference(n, k, p, q)
    print 'the difference :', r
    q.sort(reverse=True)
    r.sort(reverse=True)
    print 'the partitions, sorted :', p, q, r
    print 'checking the sum ...'
    check_sum(n, k, p, q, r)
    return p, q, r

def write_triplet(n, k, p, q):
    """
    Given a partition p and its complement q, computes the difference,
    and checks the sum.
    """
    r = difference(n, k, p, q)
    q.sort(reverse=True)
    r.sort(reverse=True)
    # print 'the triplet :', p, q, r
    bp = partition2bracket(n, k, p)
    bq = partition2bracket(n, k, q)
    br = partition2bracket(n, k, r)
    rc = resolve(n, k, [bp, bq, br], verbose=False)
    print 'the brackets :', bp, bq, br, 'root count :', rc

def write(n, k, L):
    """
    Simple callback function for generate_all_partitions.
    """
    # print 'n =', n, 'k =', k, 'L =', L
    # generate_all_complements(n, k, L, write_triplet)
    generate_all_complements(n, k, L, generate_all_differences)

def main():
    """
    Prompts the user for n, k, and m.
    """
    n = input('Give the ambient dimension : ')
    k = input('Give the dimension of the planes : ')
    m = 3 # input('Give the number of conditions : ')
    print '-> the parameters :', n, k, m
    ans = raw_input('generate all partitions ? (y/n) ')
    if(ans == 'y'):
        generate_all_partitions(n, k, write)
    else:
        p, q, r = random_triplet(n, k)
        bp = partition2bracket(n, k, p)
        bq = partition2bracket(n, k, q)
        br = partition2bracket(n, k, r)
        print 'the bracket representation :', bp, bq, br
        rc = resolve(n,k, [bp, bq, br], verbose=False)
        print 'the root count :', rc

if __name__=="__main__": main()



import MySQLdb

class PHCSQL():
   def __init__(self):
      """
      Start SQL Datebase and Return 
      """
      from phc_config import sql_host, sql_user, sql_passwd, sql_datebase
      
      self.db = MySQLdb.connect(sql_host, sql_user, sql_passwd, sql_datebase)
      self.d = self.db.cursor()
      
   def try_excute(self, commd):
      try:
         self.d.execute(commd)
      except MySQLdb.Error,e:
         print "# SQL Error = %s, %s"%(e[0],e[1])
         self.d.close()
         self.db.close()
         self.__init__()
         self.d.execute(commd)
   
   def run_commd(self, commd):
      self.try_excute(commd)
      self.db.commit()
      return 0

   def query(self,commd):
      self.try_excute(commd)
      s = self.d.fetchall()
#      self.db.commit()
      return s
   
   def close(self):
      self.db.close()

 ##################### account management #######################
   def userstatus(self, Uid, Status):
      commd = "Update users SET Status = %s WHERE Folder = '%s';" %(Status, Uid)
      self.run_commd(commd)
      return 0
      
   def check_userstatus(self, Uid):
      commd = "SELECT Status FROM users WHERE Folder='%s'"%(Uid)
      self.d.execute(commd)
      s = self.d.fetchall()
      Status = s[0][0]
      return Status
      
   def check_poly_status(self, Uid, poly_id):
      commd = "SELECT Status FROM polys WHERE Uid='%s' and Name='%s'"%(Uid, poly_id)
      self.d.execute(commd)
      s = self.d.fetchall()
      if len(s)==0:
         Status = 9
      else:
         Status = s[0][0]
      
      return Status

   def check(self, usermail, userpwd):
      """
      Check usermail and userpwd.
      If match, return the user's Folder and Name_First
      """
      error = 0
      Folder = ''
      Name_First = ''
      Status = None
      
      if len(userpwd) != 40:
         import sha
         userpwd = sha.new(userpwd).hexdigest()

      commd = "SELECT '" + userpwd + \
         "'= passwd FROM users WHERE Email='" + usermail +"'"
      self.d.execute(commd)
      s = self.d.fetchall()

      if len(s)==0 or s[0][0]==0:
         error = 4
      else:
         commd = "SELECT Folder, Name_First, Status FROM users WHERE Email='" + usermail +"'"
         self.d.execute(commd)
         s = self.d.fetchall()
         Folder = str(s[0][0])
         Name_First = str(s[0][1])
         Status = s[0][2]
         
      return error, Folder, Name_First, Status 
      
##################### polynomials management #######################
   def newpoly(self, Uid, Name, Deg, Ctime, Status='1'):
      # pop a windows to let user check same name polynomial
#      commd = "DELETE FROM polys WHERE Uid = '%s' and Name = '%s';"%(Uid,Name)
#      self.run_commd(commd)
      from datetime import datetime
      file_ctime = datetime.fromtimestamp(Ctime)
      commd = "INSERT INTO polys (Uid,Name, Deg, Ctime, Status) VALUES ('%s','%s','%d','%s','%s');" %(Uid, Name,Deg,file_ctime,Status)
      self.run_commd(commd)
      return 0

   def killpoly(self,Uid, poly_id):
      commd = "Update polys SET Status = 0 WHERE Uid = '%s' and NAME = '%s';" %(Uid, poly_id)
      self.run_commd(commd)
      return 0

   def slvpoly(self, Uid, Name):
      from phc_file import Sols, Slvtime
      sols = Sols(Uid, Name)
      slvtime = Slvtime(Uid, Name)
      commd = "Update polys SET Status = 2, Sols = %d, Time ='%s' WHERE Uid = '%s' and Name = '%s';" %(sols, slvtime, Uid, Name)
      self.run_commd(commd)
      return 0

   def slvpoly2(self, Uid, poly_id):
      from phc_file import Deg_Sols, solve_time, get_file_id
      file_id = get_file_id(Uid, poly_id)
      sols, deg = Deg_Sols(file_id)
      slvtime = solve_time(file_id)
      commd = "Update polys SET Status = 2, Sols = %d, Time ='%s' WHERE Uid = '%s' and Name = '%s';" %(sols, slvtime, Uid, poly_id)
      self.run_commd(commd)
      return 0

   def renamepoly(self, Uid, Name, NewName):
      commd = "Update polys SET Name = '%s' WHERE Uid = '%s' and Name = '%s';" %(NewName, Uid, Name)
      self.run_commd(commd)
      return 0
      
   def delpoly(self,Uid,Name):
      commd = "DELETE FROM polys WHERE Uid = '%s' and Name = '%s';" %(Uid, Name)
      self.run_commd(commd)
      return 0
      
   def allpoly(self,Uid):
      query = "SELECT * FROM polys WHERE Uid = '%s' ORDER BY Polyid DESC;" %(Uid)
      s = self.query(query)
      return s
 
def StartSQL():
   """
   Start SQL Datebase and Return 
   """
   import MySQLdb
   from phc_config import sql_host, sql_user, sql_passwd, sql_datebase

   db = MySQLdb.connect(sql_host, sql_user, sql_passwd, sql_datebase)
   d = db.cursor()
   return d,db 
 
def sql_reg(infos):
   """
   Processes password of Email form.
   """
   import sha, datetime
   from time import time
   from phc_email import Activate_Gmail
   d,db = StartSQL()
   np = sha.new(infos[1]).hexdigest()
   commd = "SELECT Uid FROM users WHERE Email='" + infos[0] +"'"
   d.execute(commd)
   s = d.fetchall()
   if len(s)==0:
      ticket = sha.new(str(time())).hexdigest()
      error = Activate_Gmail(infos[0], infos[3] , ticket)
      if error:
         db.close()
         return 4,"" # Email address is not right
      else:
         commd = "INSERT INTO users VALUES(NULL,'%s','%s','%s','%s','%s','%s','%s','%s',NULL)" % (infos[3],infos[4],infos[0],infos[5],str(np),str(datetime.date.today()),ticket,"")
         d.execute(commd)
         s = d.fetchall()
         db.commit()
         db.close()
         return 0, ticket
   else:
      db.close()
      return 3, ""

def sql_activate(email, ticket):
   """
   Processes email and ticket and generate the folder for the user.
   """
   import os, sha, time
   d,db = StartSQL()
   commd = "SELECT '%s'= Ticket FROM users WHERE Email='%s'" %(ticket, email)
   d.execute(commd)
   s = d.fetchall()
   if len(s)==0 or s[0][0]==0:
      return 2
   commd = "SELECT Folder FROM users WHERE Email='%s'" % email
   d.execute(commd)
   s = d.fetchall()
   if s[0][0] =="":
      Folder = sha.new(str(time.time())).hexdigest()
      from phc_client import PHC_Server
      PHC_Server(Folder,'3',Folder)
      commd = "UPDATE users SET Folder = '%s' WHERE Email = '%s'" % (Folder, email)
      d.execute(commd)
      s = d.fetchall()
      db.commit()
      db.close()
      return 0

   elif len(s[0][0]) == 40:
      db.close()
      Upath="../users/" + s[0][0]   
      Open_Folder = os.path.isdir(Upath)
      if Open_Folder:    
         return 3 # The account has been activated, and the folder exists.
      else:
         return 4 # The account has been activated, but the folder disappears.
   else:
      db.close()
      return 5 # The Folder data in SQL is wrong.
      
def sql_forgetpwd(Email_Address, Name_First, Name_Last, Org):
   import sha, time
   from phc_email import Resetpwd_Gmail
   
   d,db = StartSQL()
   error = 0
   
   commd = "SELECT Name_First='%s' AND Name_Last='%s' AND Org='%s' FROM users WHERE Email='%s'"%(Name_First, Name_Last, Org, Email_Address)
   d.execute(commd)
   
   s = d.fetchall()
   ticket = ''
   
   if len(s)==0 or s[0][0]==0:
      error = 4
   else:
      ticket = sha.new(str(time.time())).hexdigest()
      error = Resetpwd_Gmail(Email_Address, Name_First, ticket)
      if not error:
         commd = "UPDATE users SET Ticket = '%s', passwd = NULL WHERE Email = '%s';" % (ticket, Email_Address)
         d.execute(commd)
         s = d.fetchall()
         db.commit()
   
   db.close()
      
   return error

def sql_checkticket(email, ticket):
   """
   Processes email and ticket and generate the folder for the user.
   """
   import os, sha, time
   d,db = StartSQL()
   commd = "SELECT '%s'= Ticket FROM users WHERE Email='%s'" %(ticket, email)
   d.execute(commd)
   s = d.fetchall()
   if len(s)==0 or s[0][0]==0:
      return 1
   else:
      return 0


def sql_resetpwd(ticket, pwd):
   import sha, time
   pwd = sha.new(pwd).hexdigest()
   
   d,db = StartSQL()
   new_ticket = sha.new(str(time.time())).hexdigest()
   commd = "UPDATE users SET passwd = '%s', Ticket = '%s' WHERE Ticket = '%s';" % (pwd, new_ticket, ticket)
   d.execute(commd)
   s = d.fetchall()
   db.commit()
   
   return 0




"""
Problems for the Littlewood-Richardson homotopies.
"""
def verify(pols, sols):
    """
    Verifies whether the solutions in sols
    satisfy the polynomials of the system in pols.
    """
    from phcpy.solutions import strsol2dict, evaluate
    dictsols = [strsol2dict(sol) for sol in sols]
    checksum = 0
    for sol in dictsols:
        sumeval = sum(evaluate(pols, sol))
        print(sumeval)
        checksum = checksum + sumeval
    print('the total check sum :', checksum)

def test_basic(prc='d'):
    """
    Performs a basic test on the Littlewood-Richardson homotopies.
    """
    from phcpy.schubert import littlewood_richardson_homotopies as lrhom
    brk = [[2, 4, 6], [2, 4, 6], [2, 4, 6]]
    print('basic test with brackets', brk)
    (roco, flags, fsys, sols) = lrhom(6, 3, brk, precision=prc)
    print('the root count :', roco)
    print('the flags :', flags)
    print('the solutions :')
    for sol in sols:
        print(sol)
    verify(fsys, sols)
    print('number of solutions :', len(sols))

def test_p1(prc='d'):
    """
    First big problem in G(4,8).
    """
    from phcpy.schubert import littlewood_richardson_homotopies as lrhom
    brk = [[3, 6, 7, 8], [3, 6, 7, 8], [3, 6, 7, 8], [3, 6, 7, 8], \
        [3, 6, 7, 8], [3, 6, 7, 8], [3, 6, 7, 8], [4, 6, 7, 8], \
        [4, 6, 7, 8]]
    print('basic test with brackets', brk)
    (roco, flags, fsys, sols) = lrhom(8, 4, brk, precision=prc)
    print('the root count :', roco)
    print('the flags :', flags)
    print('the solutions :')
    for sol in sols:
        print(sol)
    verify(fsys, sols)
    print('number of solutions :', len(sols))

def main():
    """
    Tests the Littlewood-Richardson homotopies.
    """
    print("\nTesting the Littlewood-Richardson homotopies ...")
    test_basic()
    test_p1()

if __name__ == "__main__":
    main()
